@@ -17,49 +17,41 @@
 #include <string>
 #include <vector>
 
+#include "absl/strings/string_view.h"
+#include "api/environment/environment.h"
 #include "api/network_state_predictor.h"
 #include "api/sequence_checker.h"
+#include "api/task_queue/task_queue_base.h"
+#include "api/task_queue/task_queue_factory.h"
 #include "api/transport/network_control.h"
 #include "api/units/data_rate.h"
 #include "call/rtp_bitrate_configurator.h"
+#include "call/rtp_transport_config.h"
 #include "call/rtp_transport_controller_send_interface.h"
 #include "call/rtp_video_sender.h"
 #include "modules/congestion_controller/rtp/control_handler.h"
 #include "modules/congestion_controller/rtp/transport_feedback_adapter.h"
 #include "modules/congestion_controller/rtp/transport_feedback_demuxer.h"
-#include "modules/pacing/paced_sender.h"
 #include "modules/pacing/packet_router.h"
 #include "modules/pacing/rtp_packet_pacer.h"
 #include "modules/pacing/task_queue_paced_sender.h"
-#include "modules/utility/include/process_thread.h"
+#include "modules/rtp_rtcp/include/rtp_rtcp_defines.h"
 #include "rtc_base/network_route.h"
 #include "rtc_base/race_checker.h"
 #include "rtc_base/task_queue.h"
 #include "rtc_base/task_utils/repeating_task.h"
 
 namespace webrtc {
-class Clock;
 class FrameEncryptorInterface;
 class RtcEventLog;
 
-// TODO(nisse): When we get the underlying transports here, we should
-// have one object implementing RtpTransportControllerSendInterface
-// per transport, sharing the same congestion controller.
 class RtpTransportControllerSend final
     : public RtpTransportControllerSendInterface,
-      public RtcpBandwidthObserver,
+      public NetworkLinkRtcpObserver,
       public TransportFeedbackObserver,
       public NetworkStateEstimateObserver {
  public:
-  RtpTransportControllerSend(
-      Clock* clock,
-      RtcEventLog* event_log,
-      NetworkStatePredictorFactoryInterface* predictor_factory,
-      NetworkControllerFactoryInterface* controller_factory,
-      const BitrateConstraints& bitrate_config,
-      std::unique_ptr<ProcessThread> process_thread,
-      TaskQueueFactory* task_queue_factory,
-      const WebRtcKeyValueConfig& trials);
+  explicit RtpTransportControllerSend(const RtpTransportConfig& config);
   ~RtpTransportControllerSend() override;
 
   RtpTransportControllerSend(const RtpTransportControllerSend&) = delete;
@@ -83,7 +75,8 @@ class RtpTransportControllerSend final
       RtpVideoSenderInterface* rtp_video_sender) override;
 
   // Implements RtpTransportControllerSendInterface
-  rtc::TaskQueue* GetWorkerQueue() override;
+  void RegisterSendingRtpStream(RtpRtcpInterface& rtp_module) override;
+  void DeRegisterSendingRtpStream(RtpRtcpInterface& rtp_module) override;
   PacketRouter* packet_router() override;
 
   NetworkStateEstimateObserver* network_state_estimate_observer() override;
@@ -91,16 +84,18 @@ class RtpTransportControllerSend final
   RtpPacketSender* packet_sender() override;
 
   void SetAllocatedSendBitrateLimits(BitrateAllocationLimits limits) override;
+  void ReconfigureBandwidthEstimation(
+      const BandwidthEstimationSettings& settings) override;
 
   void SetPacingFactor(float pacing_factor) override;
   void SetQueueTimeLimit(int limit_ms) override;
   StreamFeedbackProvider* GetStreamFeedbackProvider() override;
   void RegisterTargetTransferRateObserver(
       TargetTransferRateObserver* observer) override;
-  void OnNetworkRouteChanged(const std::string& transport_name,
+  void OnNetworkRouteChanged(absl::string_view transport_name,
                              const rtc::NetworkRoute& network_route) override;
   void OnNetworkAvailability(bool network_available) override;
-  RtcpBandwidthObserver* GetBandwidthObserver() override;
+  NetworkLinkRtcpObserver* GetRtcpObserver() override;
   int64_t GetPacerQueuingDelayMs() const override;
   absl::optional<Timestamp> GetFirstPacketTime() const override;
   void EnablePeriodicAlrProbing(bool enable) override;
@@ -117,113 +112,105 @@ class RtpTransportControllerSend final
   void IncludeOverheadInPacedSender() override;
   void EnsureStarted() override;
 
-  // Implements RtcpBandwidthObserver interface
-  void OnReceivedEstimatedBitrate(uint32_t bitrate) override;
-  void OnReceivedRtcpReceiverReport(const ReportBlockList& report_blocks,
-                                    int64_t rtt,
-                                    int64_t now_ms) override;
+  // Implements NetworkLinkRtcpObserver interface
+  void OnReceiverEstimatedMaxBitrate(Timestamp receive_time,
+                                     DataRate bitrate) override;
+  void OnReport(Timestamp receive_time,
+                rtc::ArrayView<const ReportBlockData> report_blocks) override;
+  void OnRttUpdate(Timestamp receive_time, TimeDelta rtt) override;
+  void OnTransportFeedback(Timestamp receive_time,
+                           const rtcp::TransportFeedback& feedback) override;
 
   // Implements TransportFeedbackObserver interface
   void OnAddPacket(const RtpPacketSendInfo& packet_info) override;
-  void OnTransportFeedback(const rtcp::TransportFeedback& feedback) override;
 
   // Implements NetworkStateEstimateObserver interface
   void OnRemoteNetworkEstimate(NetworkStateEstimate estimate) override;
 
  private:
-  struct PacerSettings {
-    explicit PacerSettings(const WebRtcKeyValueConfig& trials);
-
-    bool use_task_queue_pacer() const { return !tq_disabled.Get(); }
-
-    FieldTrialFlag tq_disabled;  // Kill-switch not normally used.
-    FieldTrialParameter<TimeDelta> holdback_window;
-    FieldTrialParameter<int> holdback_packets;
-  };
-
-  void MaybeCreateControllers() RTC_RUN_ON(task_queue_);
+  void MaybeCreateControllers() RTC_RUN_ON(sequence_checker_);
+  void UpdateNetworkAvailability() RTC_RUN_ON(sequence_checker_);
   void UpdateInitialConstraints(TargetRateConstraints new_contraints)
-      RTC_RUN_ON(task_queue_);
+      RTC_RUN_ON(sequence_checker_);
 
-  void StartProcessPeriodicTasks() RTC_RUN_ON(task_queue_);
-  void UpdateControllerWithTimeInterval() RTC_RUN_ON(task_queue_);
+  void StartProcessPeriodicTasks() RTC_RUN_ON(sequence_checker_);
+  void UpdateControllerWithTimeInterval() RTC_RUN_ON(sequence_checker_);
 
   absl::optional<BitrateConstraints> ApplyOrLiftRelayCap(bool is_relayed);
   bool IsRelevantRouteChange(const rtc::NetworkRoute& old_route,
                              const rtc::NetworkRoute& new_route) const;
   void UpdateBitrateConstraints(const BitrateConstraints& updated);
-  void UpdateStreamsConfig() RTC_RUN_ON(task_queue_);
-  void OnReceivedRtcpReceiverReportBlocks(const ReportBlockList& report_blocks,
-                                          int64_t now_ms)
-      RTC_RUN_ON(task_queue_);
-  void PostUpdates(NetworkControlUpdate update) RTC_RUN_ON(task_queue_);
-  void UpdateControlState() RTC_RUN_ON(task_queue_);
-  void UpdateCongestedState() RTC_RUN_ON(task_queue_);
-  RtpPacketPacer* pacer();
-  const RtpPacketPacer* pacer() const;
+  void UpdateStreamsConfig() RTC_RUN_ON(sequence_checker_);
+  void PostUpdates(NetworkControlUpdate update) RTC_RUN_ON(sequence_checker_);
+  void UpdateControlState() RTC_RUN_ON(sequence_checker_);
+  void UpdateCongestedState() RTC_RUN_ON(sequence_checker_);
+  absl::optional<bool> GetCongestedStateUpdate() const
+      RTC_RUN_ON(sequence_checker_);
+  void ProcessSentPacket(const rtc::SentPacket& sent_packet)
+      RTC_RUN_ON(sequence_checker_);
+  void ProcessSentPacketUpdates(NetworkControlUpdate updates)
+      RTC_RUN_ON(sequence_checker_);
 
-  Clock* const clock_;
-  RtcEventLog* const event_log_;
-  SequenceChecker main_thread_;
+  const Environment env_;
+  SequenceChecker sequence_checker_;
+  TaskQueueBase* task_queue_;
   PacketRouter packet_router_;
   std::vector<std::unique_ptr<RtpVideoSenderInterface>> video_rtp_senders_
-      RTC_GUARDED_BY(&main_thread_);
+      RTC_GUARDED_BY(&sequence_checker_);
   RtpBitrateConfigurator bitrate_configurator_;
-  std::map<std::string, rtc::NetworkRoute> network_routes_;
-  bool pacer_started_;
-  const std::unique_ptr<ProcessThread> process_thread_;
-  const PacerSettings pacer_settings_;
-  std::unique_ptr<PacedSender> process_thread_pacer_;
-  std::unique_ptr<TaskQueuePacedSender> task_queue_pacer_;
+  std::map<std::string, rtc::NetworkRoute> network_routes_
+      RTC_GUARDED_BY(sequence_checker_);
+  BandwidthEstimationSettings bwe_settings_ RTC_GUARDED_BY(sequence_checker_);
+  bool pacer_started_ RTC_GUARDED_BY(sequence_checker_);
+  TaskQueuePacedSender pacer_;
 
-  TargetTransferRateObserver* observer_ RTC_GUARDED_BY(task_queue_);
+  TargetTransferRateObserver* observer_ RTC_GUARDED_BY(sequence_checker_);
   TransportFeedbackDemuxer feedback_demuxer_;
 
   TransportFeedbackAdapter transport_feedback_adapter_
-      RTC_GUARDED_BY(task_queue_);
+      RTC_GUARDED_BY(sequence_checker_);
 
   NetworkControllerFactoryInterface* const controller_factory_override_
-      RTC_PT_GUARDED_BY(task_queue_);
+      RTC_PT_GUARDED_BY(sequence_checker_);
   const std::unique_ptr<NetworkControllerFactoryInterface>
-      controller_factory_fallback_ RTC_PT_GUARDED_BY(task_queue_);
+      controller_factory_fallback_ RTC_PT_GUARDED_BY(sequence_checker_);
 
   std::unique_ptr<CongestionControlHandler> control_handler_
-      RTC_GUARDED_BY(task_queue_) RTC_PT_GUARDED_BY(task_queue_);
+      RTC_GUARDED_BY(sequence_checker_) RTC_PT_GUARDED_BY(sequence_checker_);
 
   std::unique_ptr<NetworkControllerInterface> controller_
-      RTC_GUARDED_BY(task_queue_) RTC_PT_GUARDED_BY(task_queue_);
+      RTC_GUARDED_BY(sequence_checker_) RTC_PT_GUARDED_BY(sequence_checker_);
 
-  TimeDelta process_interval_ RTC_GUARDED_BY(task_queue_);
+  TimeDelta process_interval_ RTC_GUARDED_BY(sequence_checker_);
 
-  std::map<uint32_t, RTCPReportBlock> last_report_blocks_
-      RTC_GUARDED_BY(task_queue_);
-  Timestamp last_report_block_time_ RTC_GUARDED_BY(task_queue_);
+  struct LossReport {
+    uint32_t extended_highest_sequence_number = 0;
+    int cumulative_lost = 0;
+  };
+  std::map<uint32_t, LossReport> last_report_blocks_
+      RTC_GUARDED_BY(sequence_checker_);
+  Timestamp last_report_block_time_ RTC_GUARDED_BY(sequence_checker_);
 
-  NetworkControllerConfig initial_config_ RTC_GUARDED_BY(task_queue_);
-  StreamsConfig streams_config_ RTC_GUARDED_BY(task_queue_);
+  NetworkControllerConfig initial_config_ RTC_GUARDED_BY(sequence_checker_);
+  StreamsConfig streams_config_ RTC_GUARDED_BY(sequence_checker_);
 
   const bool reset_feedback_on_route_change_;
-  const bool send_side_bwe_with_overhead_;
   const bool add_pacing_to_cwin_;
   FieldTrialParameter<DataRate> relay_bandwidth_cap_;
 
-  size_t transport_overhead_bytes_per_packet_ RTC_GUARDED_BY(task_queue_);
-  bool network_available_ RTC_GUARDED_BY(task_queue_);
-  RepeatingTaskHandle pacer_queue_update_task_ RTC_GUARDED_BY(task_queue_);
-  RepeatingTaskHandle controller_task_ RTC_GUARDED_BY(task_queue_);
+  size_t transport_overhead_bytes_per_packet_ RTC_GUARDED_BY(sequence_checker_);
+  bool network_available_ RTC_GUARDED_BY(sequence_checker_);
+  RepeatingTaskHandle pacer_queue_update_task_
+      RTC_GUARDED_BY(sequence_checker_);
+  RepeatingTaskHandle controller_task_ RTC_GUARDED_BY(sequence_checker_);
 
-  DataSize congestion_window_size_ RTC_GUARDED_BY(task_queue_);
-  bool is_congested_ RTC_GUARDED_BY(task_queue_);
+  DataSize congestion_window_size_ RTC_GUARDED_BY(sequence_checker_);
+  bool is_congested_ RTC_GUARDED_BY(sequence_checker_);
 
   // Protected by internal locks.
   RateLimiter retransmission_rate_limiter_;
 
-  // TODO(perkj): `task_queue_` is supposed to replace `process_thread_`.
-  // `task_queue_` is defined last to ensure all pending tasks are cancelled
-  // and deleted before any other members.
-  rtc::TaskQueue task_queue_;
-
-  const WebRtcKeyValueConfig& field_trials_;
+  ScopedTaskSafety safety_;
 };
 
 }  // namespace webrtc
