@@ -20,11 +20,10 @@ namespace webrtc {
 // controller.
 class RtpTransportControllerSendFactoryInterface {
  public:
-  virtual std::unique_ptr<RtpTransportControllerSendInterface> Create(
-      const RtpTransportConfig& config,
-      Clock* clock) = 0;
+  virtual ~RtpTransportControllerSendFactoryInterface() = default;
 
-  virtual ~RtpTransportControllerSendFactoryInterface() {}
+  virtual std::unique_ptr<RtpTransportControllerSendInterface> Create(
+      const RtpTransportConfig& config) = 0;
 };
 }  // namespace webrtc
 #endif  // CALL_RTP_TRANSPORT_CONTROLLER_SEND_FACTORY_INTERFACE_H_
